#
#  AppController.rb
#  Transcriptize
#
#  Created by Gregory Clarke on 1/22/10.
#  Copyright (c) 2010 Intelligent Assistance, Inc. All rights reserved.
#

require 'osx/cocoa'
require 'transcriptase_algorithm'

class AppController < OSX::NSObject
	ib_outlets :mainWindow, :deactivateMenuItem, :saveMenuItem
	ib_outlets :transcriptizeView, :movIcon1, :movFileName1, :wordGroupingSlider, :identifySpeakersButton, :openSpreadsheetCheckbox, :saveTextButton, :saveExcelButton, :spinner1, :chooseXMLButton2, :xmlIcon2, :xmlFileName2, :saveXMLButton, :spinner2
	ib_outlets :speakersPanel, :speakersTableView
	ib_outlets :regoView, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton
	
	def awakeFromNib
		@mainWindow.contentView.wantsLayer = true
		@webView.setDrawsBackground(false)
		@movIcon1.setHidden(true)
		@movIcon1.setToolTip("")
		@movFileName1.setStringValue("")
		@transcription_mov = nil
		@xmlIcon2.setHidden(true)
		@xmlIcon2.setToolTip("")
		@xmlFileName2.setStringValue("")
		@clip_xml = nil
		@timecodereader = OSX::NSBundle.mainBundle.resourcePath.fileSystemRepresentation + "/third-party/timecodereader"
		@xmpcommand = OSX::NSBundle.mainBundle.resourcePath.fileSystemRepresentation + "/third-party/xmpcommand"
	end
	
	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		puts "applicationWillFinishLaunching"
		check_for_expired_beta()
		if @this_rego[0]["active"]
			showTranscriptizeView()
		elsif @this_rego[0]["mac_problem"]==true
			check_for_rego(@this_rego)
		else
			@deactivateMenuItem.setTitle("Activate...")
			displayDemoSheet()
		end
	end
	
	def displayDemoSheet
		alert = OSX::NSAlert.alloc.init
		alert.setMessageText("#{SKU} has not been activated for this computer.")
		alert.setAlertStyle(OSX::NSInformationalAlertStyle)
		alert.setInformativeText("Would you like to activate #{SKU}, or use it in demo mode  (which has a limited transcription)?")
		alert.addButtonWithTitle("Activate")
		alert.addButtonWithTitle("Buy")
		alert.addButtonWithTitle("Demo")
		alert.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, "didEndDemoSheet:returnCode:contextInfo:", nil)
	end

	def didEndDemoSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		sheet.window.orderOut(nil)
		if buttonClicked==OSX::NSAlertSecondButtonReturn
			system('open', 'http://www.AssistedEditing.com/Transcriptize/buy')
		end
		if buttonClicked==OSX::NSAlertThirdButtonReturn
			showTranscriptizeView()
		else
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC)
		end
	end

	def showDeactivation
		if @deactivateMenuItem.title=="Activate..."
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			newFrame = self.newFrameForNewContentView(@deactivateAreaView) # 500 x 200
			@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setContentView(@deactivateAreaView)
			showActivationView(newFrame, WEB_APP + "destroy/" + @this_rego[0]["number"].to_s)
		end
		@closeRegoButton.setTitle("Cancel")
	end
	ib_action :showDeactivation # Transcriptize > Activate or Deactivate
	
	def showActivationView(frame, url)
		puts "showActivationView #{url}"
		@mainWindow.setFrame_display frame, true
		@mainWindow.setContentView(@regoView)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		puts notification.name
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			showTranscriptizeView()
		else
			OSX::NSApplication.sharedApplication.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def showTranscriptizeView
		puts "showTranscriptizeView"
		setupView(@transcriptizeView, "Transcriptize", 500, 400)
	end

	def setupView(view, title, width, height)
		@mainWindow.setTitle(title)
		oldFrame = @mainWindow.frame
		newFrame = self.newFrameForNewContentView(view)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setFrame_display_animate(newFrame, true, true)
		@mainWindow.setContentView(view)
	end

	def newFrameForNewContentView(view)
		newFrameRect = @mainWindow.frameRectForContentRect(view.frame)
		oldFrameRect = @mainWindow.frame
		newSize = newFrameRect.size
		oldSize = oldFrameRect.size
		frame = @mainWindow.frame
		frame.size = newSize
		frame.origin.y = frame.origin.y - (newSize.height - oldSize.height)
		frame
	end

	def showHelp(sender)
		openPDF("Transcriptize - Help.pdf")
	end
	ib_action :showHelp # Help > Transcriptize Help
	
	def openPDF(filename)
		path_to_file = __FILE__.sub("AppController.rb",filename)
		system('open', path_to_file)
	end
	
	def visitWebsite(sender)
		system('open', 'http://www.AssistedEditing.com/')
	end
	ib_action :visitWebsite # Help > Visit www.AssistedEditing.com
	
	def sendFeedback(sender)
		OSX::JRFeedbackController.showFeedback
	end
	ib_action :sendFeedback # Transcriptize > Provide Feedback

	def sendSupportEmail(subject)
		system('open', "mailto:techsupport@intelligentassistance.com?subject=Transcriptize #{subject}&body=Attached is the XML file(s) that Transcriptize was unable to process.")
	end

	def chooseMovFile1(sender)
		@transcription_xml = nil
		@movIcon1.setHidden(true)
		@movIcon1.setToolTip("")
		@movFileName1.setStringValue("")
		filetypes = ["mov"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setMessage("Choose the QuickTime movie with speech transcription")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndOpenSheet1:returnCode:contextInfo:', nil)
	end
	ib_action :chooseMovFile1
	
	def didEndOpenSheet1_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			file = oPanel.URL.path.to_s
			application_setMovFile1(self, file)
		end
	end
	
	def application_setMovFile1(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		fileManager = OSX::NSFileManager.defaultManager
		@mov_file_path = path.to_s
		@spinner1.startAnimation(self)
		parse_xmp_from_mov(@mov_file_path) # sets @transcription_xml
		@spinner1.stopAnimation(self)
		unless @transcription_xml.nil?
			@mov_file_name = File.basename(@mov_file_path)
			@movIcon1.setHidden(false)
			@movIcon1.setToolTip(path.to_s)
			@movFileName1.setStringValue(@mov_file_name)
			@movIcon1.setNeedsDisplay(true)
			@movFileName1.setNeedsDisplay(true)
			build_speakers_array()
			@identifySpeakersButton.setEnabled(true)
			@saveTextButton.setEnabled(true)
			@saveExcelButton.setEnabled(true)
			@chooseXMLButton2.setEnabled(true)
			@saveXMLButton.setEnabled(true) unless @clip_xml.nil?
		else
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("This movie has no speech transcription")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			alert.setInformativeText("Please choose a QuickTime movie that has been transcribed using Soundbooth or Premiere Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
			chooseMovFile1(nil)
		end
	end
	
	def identifySpeakers(sender)
		puts "***identifySpeakers", @speakers_array.inspect
		@speakersTableView.reloadData
		@speakersPanel.makeKeyAndOrderFront(self)
	end
	ib_action :identifySpeakers
	
	def wordGroupingSlider_value
		tick_value = @wordGroupingSlider.floatValue.to_i
		case
		when tick_value == 11
			return 100
		when tick_value <= 5
			return tick_value
		when tick_value > 5
			return (tick_value - 5) * 10
		end
	end
	
	def saveText(sender)
		puts @speakers_array.inspect
		sPanel = OSX::NSSavePanel.savePanel
		sPanel.setMessage("Save the Text file")
		sPanel.setRequiredFileType("txt")
		sPanel.setExtensionHidden(false)
		sPanel.setCanSelectHiddenExtension(true)
		sPanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, @mov_file_name.sub(".mov",".txt"), @mainWindow, self, 'didEndSaveText:returnCode:contextInfo:', nil)
	end
	ib_action :saveText
	
	def didEndSaveText_returnCode_contextInfo(sPanel, buttonClicked, info)
		@spinner1.startAnimation(self)
		########## MAKE AND SAVE TEXT ##########
		if buttonClicked == OSX::NSOKButton
			sPanel.orderOut(self)
			if sPanel.isExtensionHidden?
				extension_hidden = 1
			else
				extension_hidden = 0
			end
			puts "is extension hidden?", sPanel.isExtensionHidden
			text_file = sPanel.URL.path.to_s
			begin
				save_to_text(wordGroupingSlider_value, text_file)
			rescue
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem saving a text file of this transcription.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with saveText()"
				alert.setInformativeText("Please choose a QuickTime movie that has been transcribed using Soundbooth or Premiere Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
				chooseMovFile1(nil)
			end
		end
		@spinner1.stopAnimation(self)
	end
	
	def saveExcel(sender)
		puts @speakers_array.inspect
		sPanel = OSX::NSSavePanel.savePanel
		sPanel.setMessage("Save the Excel spreadsheet")
		sPanel.setRequiredFileType("xls")
		sPanel.setExtensionHidden(false)
		sPanel.setCanSelectHiddenExtension(true)
		sPanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, @mov_file_name.sub(".mov",".xls"), @mainWindow, self, 'didEndSaveExcel:returnCode:contextInfo:', nil)
	end
	ib_action :saveExcel
	
	def didEndSaveExcel_returnCode_contextInfo(sPanel, buttonClicked, info)
		@spinner1.startAnimation(self)
		########## MAKE AND SAVE EXCEL ##########
		if buttonClicked == OSX::NSOKButton
			sPanel.orderOut(self)
			if sPanel.isExtensionHidden?
				extension_hidden = 1
			else
				extension_hidden = 0
			end
			puts "is extension hidden?", sPanel.isExtensionHidden
			excel_file = sPanel.URL.path.to_s
			begin
				save_to_excel(wordGroupingSlider_value, excel_file)
				if @openSpreadsheetCheckbox.state==1
					OSX::NSWorkspace.sharedWorkspace.openFile(excel_file)
				end
			rescue
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem saving an Excel spreadsheet of this transcription.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with saveExcel()"
				alert.setInformativeText("Please choose a QuickTime movie that has been transcribed using Soundbooth or Premiere Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
				chooseMovFile1(nil)
			end
		end
		@spinner1.stopAnimation(self)
	end

	def chooseXmlFile2(sender)
		@clip_xml = nil
		@xmlIcon2.setHidden(true)
		@xmlIcon2.setToolTip("")
		@xmlFileName2.setStringValue("")
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setTitle("Choose the exported clip (XML file)")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndOpenSheet2:returnCode:contextInfo:', nil)
	end
	ib_action :chooseXmlFile2
	
	def didEndOpenSheet2_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			file = oPanel.URL.path.to_s
			application_setXmlFile2(self, file)
		end
	end
	
	def application_setXmlFile2(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		fileManager = OSX::NSFileManager.defaultManager
		chosen_file = path.to_s
		@spinner1.startAnimation(self)
		@clip_xml, doctype = parse_xml_file(chosen_file)
		@spinner1.stopAnimation(self)
		if doctype=="xmeml"
			@xmlIcon2.setHidden(false)
			@xmlIcon2.setToolTip(path.to_s)
			@xmlFileName2.setStringValue(File.basename(chosen_file))
			@xmlIcon2.setNeedsDisplay(true)
			@xmlFileName2.setNeedsDisplay(true)
			@clip_xml_clone = @clip_xml.clone
			unless @transcription_mov.nil?
				@identifySpeakersButton.setEnabled(true) 
				@saveTextButton.setEnabled(true)
				@saveExcelButton.setEnabled(true)
			end
			@saveXMLButton.setEnabled(true)
		else
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("This is not a Clip XML file")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			alert.setInformativeText("Please choose a Clip XML file exported from\nFinal Cut Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
			chooseXmlFile2(nil)
		end
	end

	def saveClipXML(sender)
		puts @speakers_array.inspect
		@spinner2.startAnimation(self)
		begin
			create_clip_with_markers()
		rescue
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem adding markers to this clip.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			puts "RESCUE!!! Problem with saveExcel()"
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
			chooseXmlFile2(nil)
		end
		@spinner2.stopAnimation(self)
	end
	ib_action :saveClipXML

	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end

	### TableView methods ###

	def numberOfRowsInTableView(tableView)
		if @speakers_array.nil?
			return 0
		else
			return @speakers_array.length
		end
	end

	def tableView_objectValueForTableColumn_row(tableView, tableColumn, rowIndex)
		["Speaker", "Name"].each do |column|
			if tableColumn.headerCell.stringValue == column
				object = @speakers_array[rowIndex]
				return object[column] unless object.nil?
			end
		end
	end
	
	def tableView_setObjectValue_forTableColumn_row(tableView, anObject, tableColumn, rowIndex)
		tableView.deselectRow(rowIndex)
		@speakers_array[rowIndex]["Name"] = anObject.to_s
		puts @speakers_array.inspect
	end

end
